"""
Testing of the fief module.

"""

from fief import filter_effective_parameters as fief


EFFECTIVE_PARAMETERS = {
    'a': 2,
    'b': 3,
    'c': 4,
}


@fief
def funcA(a, b):
    return a + b


# Here is dragons
#  Expected error:
#      TypeError: funcA() got an unexpected keyword argument 'c'
#  But the fief decorator removes the problem:
assert funcA(**EFFECTIVE_PARAMETERS) == 5



"""
This module implements a decorator that allows a complete filtering of
effective parameters with respect to the formal ones.

"""

from inspect import signature
from functools import wraps
from collections import ChainMap



def filter_effective_parameters(func):
    """Decorator that filter out parameters in kwargs that are not related to
    any formal parameter of the given function.

    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        formal_parameters = frozenset(signature(func).parameters.keys())
        return func(*args, **{
            arg: value
            for arg, value in kwargs.items()
            if arg in formal_parameters
        })
    return wrapper



from setuptools import setup, find_packages
from fief.info import __version__, __name__


setup(
    name = __name__,
    version = __version__,
    packages = find_packages(),
    package_data = {
        __name__ : ['README.mkd', 'LICENSE.txt', 'requirements']
    },
    include_package_data = True,

    author = "lucas",
    author_email = "lucas.bourneuf@laposte.net",
    description = "Decorator for effective parameter filtering",
    long_description = open('README.mkd').read(),
    keywords = "decorator parameter filter",
    url = "https://github.com/Aluriak/fief",

    classifiers = [
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)




